@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-form-login-prefix-cls: ~'@{ant-prefix}-pro-form-login';

.@{pro-form-login-prefix-cls}-container {
  display: flex;
  flex: 1;
  flex-direction: column;
  height: 100%;
  padding: 32px 0;
  overflow: auto;
  background: inherit;
}

@media (min-width: @screen-md-min) {
  .@{pro-form-login-prefix-cls}-container {
    padding: 32px 0 24px;
    background-repeat: no-repeat;
    background-position: center 110px;
    background-size: 100%;
  }
}

.@{pro-form-login-prefix-cls}-top {
  text-align: center;
}

.@{pro-form-login-prefix-cls}-header {
  display: flex;
  align-items: center;
  justify-content: center;
  height: 44px;
  line-height: 44px;
  a {
    text-decoration: none;
  }
}

.@{pro-form-login-prefix-cls}-title {
  position: relative;
  top: 2px;
  color: @heading-color;
  font-weight: 600;
  font-size: 33px;
}

.@{pro-form-login-prefix-cls}-logo {
  width: 44px;
  height: 44px;
  margin-right: 16px;
  vertical-align: top;

  img {
    width: 100%;
  }
}

.@{pro-form-login-prefix-cls}-desc {
  margin-top: 12px;
  margin-bottom: 40px;
  color: @text-color-secondary;
  font-size: @font-size-base;
}
@media screen and (max-width: @screen-sm) {
  .@{pro-form-login-prefix-cls}-main {
    width: 95%;
    max-width: 328px;
  }
}

.@{pro-form-login-prefix-cls}-main {
  width: 328px;
  margin: 0 auto;

  .@{ant-prefix}-tabs-nav-list {
    margin: 0px auto;
    font-size: 16px;
  }

  .@{pro-form-login-prefix-cls}-other {
    margin-top: 24px;
    line-height: 22px;
    text-align: left;
  }
}
