@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-form-query-filter-prefix-cls: ~'@{ant-prefix}-pro-form-query-filter';

.@{pro-form-query-filter-prefix-cls} {
  /*暂时不需要内容*/
}
