@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-core-dropdown-footer: ~'@{ant-prefix}-pro-core-dropdown-footer';

.@{pro-core-dropdown-footer} {
  display: flex;
  justify-content: space-between;
  padding: 16px 16px 16px 8px;
  border-top: 1px solid @border-color-split;
}
