@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-form-group-prefix-cls: ~'@{ant-prefix}-pro-form-group';

.@{pro-form-group-prefix-cls} {
  &-title {
    margin-bottom: @margin-md;
    font-weight: bold;
  }
  &-container {
    flex-wrap: wrap;
    max-width: 100%;
    > div.@{ant-prefix}-space-item {
      max-width: 100%;
    }
  }

  &-twoLine {
    display: block;
    width: 100%;

    .@{pro-form-group-prefix-cls}-title {
      width: 100%;
      margin: 8px 0;
    }
    .@{pro-form-group-prefix-cls}-container {
      padding-left: 16px;
    }

    .@{ant-prefix}-space-item,
    .@{ant-prefix}-form-item {
      width: 100%;
    }

    .@{ant-prefix}-form-item {
      &-control {
        display: flex;
        align-items: center;
        justify-content: flex-end;

        &-input {
          align-items: center;
          justify-content: flex-end;

          &-content {
            flex: none;
          }
        }
      }
    }
  }
}
