@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-core-field-dropdown: ~'@{ant-prefix}-pro-core-field-dropdown';

.@{pro-core-field-dropdown}-label {
  cursor: pointer;
}

.@{pro-core-field-dropdown}-overlay {
  min-width: 200px;
  margin-top: 4px;
  background-color: @component-background;
  box-shadow: @card-shadow;
}

.@{pro-core-field-dropdown}-content {
  padding: 16px;
}
